% drivefx5.m
% Drives fx floor program with 5 assets (spot + 4options)

%
clear
tic

runtype3dat = 3;   % 2 data for gold update  [for runtype=3 only] , 3 full sample           

if runtype3dat==2
        DATu = xlsread('eurCHFVOLSall.xlsx',2,'h13:o127');
elseif runtype3dat==3
        DATu = xlsread('eurCHFVOLSall.xlsx',1,'g183:o1030');
end

T=max(size(DATu));
% Implied volatilities for 25 delta and 10 delta

VPUs = [ DATu(:,2)+DATu(:,4)-.5*DATu(:,3) DATu(:,2)+DATu(:,6)-.5*DATu(:,5) ]/100;
VCAs = [ DATu(:,2)+DATu(:,4)+.5*DATu(:,3) DATu(:,2)+DATu(:,6)+.5*DATu(:,5) ]/100;
KP25 = DATu(:,1).*((1+DATu(:,7)/400)./(1+DATu(:,8)/400)).*exp( norminv(.25*(1+DATu(:,7)/400)).*VPUs(:,1)*(.25^.5)+.5*(VPUs(:,1).^2)*.25 );
KP10 = DATu(:,1).*((1+DATu(:,7)/400)./(1+DATu(:,8)/400)).*exp( norminv(.10*(1+DATu(:,7)/400)).*VPUs(:,2)*(.25^.5)+.5*(VPUs(:,2).^2)*.25 );
KC25 = DATu(:,1).*((1+DATu(:,7)/400)./(1+DATu(:,8)/400)).*exp(-norminv(.25*(1+DATu(:,7)/400)).*VCAs(:,1)*(.25^.5)+.5*(VCAs(:,1).^2)*.25 );
KC10 = DATu(:,1).*((1+DATu(:,7)/400)./(1+DATu(:,8)/400)).*exp(-norminv(.10*(1+DATu(:,7)/400)).*VCAs(:,2)*(.25^.5)+.5*(VCAs(:,2).^2)*.25 );    
    

[~,P25] = blspriceu(DATu(:,1), KP25, log(1+DATu(:,7)/100), .25, VPUs(:,1), log(1+DATu(:,8)/100));
[~,P10] = blspriceu(DATu(:,1), KP10, log(1+DATu(:,7)/100), .25, VPUs(:,2), log(1+DATu(:,8)/100));
[C25,~] = blspriceu(DATu(:,1), KC25, log(1+DATu(:,7)/100), .25, VCAs(:,1), log(1+DATu(:,8)/100));
[C10,~] = blspriceu(DATu(:,1), KC10, log(1+DATu(:,7)/100), .25, VCAs(:,2), log(1+DATu(:,8)/100));

     tpv = .25*ones(T,1);   %IF have differnent maturities need to change fxf5.m 
     %           Kp1   tp1   Kp2  tp2   Kp3  tp3   Kp4  tp4       r           rs            S          P1   P2  P3  P4
       
     PARAMmins=[ KP10  tpv   KP25 tpv  KC10  tpv   KC25  tpv  DATu(:,7)/100 DATu(:,8)/100  DATu(:,1)  P10  P25  C10 C25  ];
  
     % Step vectors for initial conditions. Start large, small at convergence
     stps= [ .04   .02  .02];
     X0s=[ 0 0 0
          stps
          -stps
          stps-2*[stps(1)   0  0]
          stps-2*[0  stps(2)   0]
          stps-2*[0   0  stps(3)]  
          stps-2*[stps(1) stps(2)  0]
          stps-2*[stps(1) 0  stps(3)] 
          stps-2*[0 stps(2) stps(3)]  ];  
     
  syp=9;      %Either 1 or 9 for multiple initial conditions
        XA1=zeros(T,9);
        XA2=zeros(T,9);
        XA3=zeros(T,9);
        DIFsA=ones(T,syp);
      
  for jj=1:syp   %Either 1 or 9 for multiple initial conditions
      
  init=1;   % 1 here, 2 load 
  
    if init==1
        X0=zeros(T,3);
        %         vi0        p0       sig0
       x01=  [   1.1497    0.8352    0.0743 ];
    for j=1:T
        X0(j,:)=x01;
    end
    elseif init==2
        if runtype3dat==2
            load inisav2
        elseif runtype3dat==3
            load inisav3
        end
    end 
     
  for jjj=1:T
      X0(jjj,:)=X0(jjj,:)+ X0s(jj,:);
  end
   
   compvec =[ 1:848 ];    % 1:848 , 1:115 \\ Can use this to run for subset of periods
   compvec =[ 845:848 ];
   
    X=zeros(T,3);
    DIFs=zeros(T,1);
            parfor j=compvec
                x = fminsearchbnd(@(x) impfxf5(x,PARAMmins(j,:)),X0(j,:),[0.8 0 0],[2  1  .5], optimset('TolX',1e-8,'TolFun',1e-10))
    
                X(j,:) = x ;
                difs = impfxf5(x,PARAMmins(j,:));
                DIFs(j,1) = difs ;  
            end
          
        XA1(:,jj)=X(:,1);
        XA2(:,jj)=X(:,2);
        XA3(:,jj)=X(:,3);
        DIFsA(:,jj)=DIFs;
        [m , inm]=min(DIFsA,[],2);
        Xbe=zeros(T,3);

    for jv=1:T
        Xbe(jv,1)=XA1(jv,inm(jv));
        Xbe(jv,2)=XA2(jv,inm(jv));
        Xbe(jv,3)=XA3(jv,inm(jv));
    end
         
  end
  
  figure(4)
    plot(inm,'.')
    sum(inm<1.5)
    
   % KEEP at upda at 0 to MANUALLY update
   %    1) Check (i) different initial wins (not converged) (ii) screw ups
   %    2) use solution as initial condition
   %                  >>      X0=Xbe;     
   %    3) give new initial conditions for outliers/screw ups
   %    4) save initial conditions for uploading
   %     if runtype3data==2 use >>  save inisav2 X0
   %     if runtype3data==3 use >>  save inisav3 X0
   
upda=0;
if upda==1
  
    %outliers/screw ups
    count=1:T;
    indi1= .9>Xbe(:,1);
    indi2= .1>Xbe(:,2);
    indi3= .25<Xbe(:,3);
    count(indi1)
    count(indi2)
    count(indi3)

    updain=0;
    if updain==1 
         indji=count(indi2)';
        
        for ji=1:max(size(count(indi2)));
           X0(indji(ji),:)= [1.17    0.6   0.12];
        end
    end
end
   
runu=1;

if runu==1
    figure(1)
    subplot(2,2,1)
    plot(m(compvec,1))
    subplot(2,2,2)
    plot(Xbe(compvec,1))
    subplot(2,2,3)
    plot(Xbe(compvec,2))
    subplot(2,2,4)
    plot(Xbe(compvec,3))
 
end    
 
toc    

