function [y,Pv,PI1] =fxf5(x,params)
% fx floor function for EUR/CHF, with 5 assets 1 fx + 4 options
% function to compute y:
% S     spot
% P1 P2 P3 P4 options typically 2 PUTs and 2 Calls

% as a function of x:
% vi     fundamental value
% p     proba of continuation of floor
% sig   risk neutral vol of exchange rate
%           1    2   3   4   5  6    7   8  9   10
% params = [Kp1 tp1 Kp2 tp2 Kp3 tp3 Kp4 tp4 r  rs ]


vi=  x(1);       % value for V where want to compute S, and option prices
p=   x(2);       % QUARTERLY proba of continuation of policy to next period
sig= x(3);       % ANNUAL standard deviation of exchange rate (log)

r = params(9);   % interest CHF annual
rs= params(10);  % interest EUR annual
K=1.20 ;         % floor

vss=1.2;         % middle of fundamental exchange rate vector
ng=50*2;         % number of steps up or down in grid

fra=1/52/2;      % /2 period length fraction of year
maxit=1000;

r=(1+r)^fra-1;
rs=(1+rs)^fra-1;
sig=sig*fra^.5;
p=p^(fra*4);

u=((1+rs)/(1+r))*exp(sig);      
d=((1+rs)/(1+r))*exp(-sig);

q=((1+r)/(1+rs)-d)/(u-d);
qle=.1;      
que=.9;       

V=zeros(2*ng+1,1);
V(ng+1)=vss;
for j=1:ng
    V(ng+1+j)=V(ng+j)*u;
    V(ng+1-j)=V(ng+1-j+1)*d;
end
Kv=K*ones(size(V));

PI=zeros(2*ng+1,2*ng+1);

PI(1,1:2)=[(1-qle) qle];
PI(2*ng+1,(2*ng):(2*ng+1))=[(1-que) que];

PI(ng+1,ng:ng+2)=[(1-q) 0 q];
for j=1:ng-1
    PI(ng+1+j,(ng+j):(ng+2+j))=[(1-q) 0 q];
    PI(ng+1-j,(ng-j):(ng+2-j))=[(1-q) 0 q];
end

S=V;
crit=ones(2*ng+1,1);
for j=1:maxit
    maS=max([S Kv],[],2);
    TS= ((1+rs)/(1+r))*p*(PI*maS ) +(1-p)*V ;
    crit(j)=max (abs(TS-S));
    S=TS;
end

%Price options
Kp1=params(1);   
tp1=params(2);   

Kp2=params(3);   
tp2=params(4);   

Kp3=params(5);   
tp3=params(6);   

Kp4=params(7);  
tp4=params(8);   

tp1=(tp1/fra);    
tp2=(tp2/fra);   
tp3=(tp3/fra);
tp4=(tp4/fra);

zerov=zeros(ng*2+1,1);

% Puts
PI1=PI^tp1;
pp1=p^tp1;


PI2=PI1;
pp2=pp1;
PI3=PI1;
pp3=pp1;
PI4=PI1;
pp4=pp1;

% Puts
P1= (1/(1+r)^tp1)*PI1*(   pp1*max([Kp1-max([S,Kv],[],2),zerov],[],2) +(1-pp1)*max([Kp1-V,zerov],[],2));
P2= (1/(1+r)^tp2)*PI2*(   pp2*max([Kp2-max([S,Kv],[],2),zerov],[],2) +(1-pp2)*max([Kp2-V,zerov],[],2));

% Calls
P3= (1/(1+r)^tp3)*PI3*(   pp3*max([max([S,Kv],[],2)-Kp3,zerov],[],2) +(1-pp3)*max([V-Kp3,zerov],[],2));
P4= (1/(1+r)^tp4)*PI4*(   pp4*max([max([S,Kv],[],2)-Kp4,zerov],[],2) +(1-pp4)*max([V-Kp4,zerov],[],2));

crit(maxit) ;

y=interp1(V,[S P1 P2 P3 P4],vi,'spline') ;    


Pv=[V,S,P1,P2,P3,P4];
plot([S V Kv])
